//	-------------------------------------------------------------------------
//
//	ACCESSRY.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the Accessory base class as
//					well as functions for installing accessory bitmaps, etc.
//					eg. InstallGUI() and ExitGUI()
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "accessry.hpp"
#include "files.hpp"
#include "strings.hpp"
#include "misc.hpp"


// the file spec, etc used to store GUI bitmaps...
#define 	GUI_BITMAP_MAX		256

File*		GUI_BITMAP_FILE	= NULL;   	// the file for the bitmaps
DirString*	GUI_BITMAP_STR	= NULL;     // the string for the file
grImage*  	GUI_BITMAP[GUI_BITMAP_MAX];	// the bitmap storage area
int			GUI_BITMAP_CNT	= 0;		// how many bitmaps we've loaded

//	-------------------------------------------------------------------------
//
//	FUNC:		InstallGUI
//	DESC:       A routine used to load the default bitmaps used by any of the
//				predefined system Accessories.
//
//	NOTES:		(1) _Before_ using any PictureButton accessory you MUST call
//					this function!
//
//				(2)	To modify the "look & feel" of the GUI, you can edit the
//					bitmaps in the file ACCESSRY.BTM _but_ you must keep the
//					same image order. ie. BLANK1, BLANK2, UP_ARR1, UP_ARR2,
//					DN_ARR1, DN_ARR2, etc.
//
//	-------------------------------------------------------------------------

void InstallGUI()
	{
	// the install simply loads relevant bitmaps for the GUI into memory
	GUI_BITMAP_STR = new DirString("ACCESSRY.BTM");
	GUI_BITMAP_FILE = new File(GUI_BITMAP_STR,file_READWRITE);
	GUI_BITMAP_FILE->Open();
	int cnt = 0;
	long siz = 0;
	while (!GUI_BITMAP_FILE->Eof())
		{
		siz = GUI_BITMAP_FILE->Recsize(cnt);
		GUI_BITMAP[cnt] = (grImage*) malloc(siz);
		GUI_BITMAP_FILE->Read(cnt,GUI_BITMAP[cnt]);
		cnt++;
		}
	GUI_BITMAP_FILE->Close();
	GUI_BITMAP_CNT = cnt;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ExitGUI
//	DESC:       This routine releases all resources used by InstallGUI!
//
//	-------------------------------------------------------------------------

void ExitGUI()
	{
	if (GUI_BITMAP_CNT!=0)
		for (int i=GUI_BITMAP_CNT;i>=0;i--)
			{
			free(GUI_BITMAP[i]); GUI_BITMAP[i] = NULL;
			}
	delete GUI_BITMAP_FILE;
	delete GUI_BITMAP_STR;
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for Accessory
//	DESC:       This is the base constructor for _all_ Accessory's.
//
//	PARAMS:		x1,y1,x2,y2			- the relative (to the handler's inset_r)
//									  coordinates of the accessory
//				fg,bg				- the foreground and background colors
//									  when inactive
//				active_border,
//				inactive_border		- the border style for active & inactive
//				options				- the options for the accessory
//				retevent			- the event the accessory returns to
//									  its eventhandler
//
//	-------------------------------------------------------------------------
Accessory::Accessory(int x1,int y1,int x2,int y2,uchar fg,uchar bg,
					 uint active_border,uint inactive_border,
					 uint options,uint retevent)
	{
	// set the default colors
	fgcolor = fg; bgcolor = bg;

	// set the border styles
	active_bs = active_border;
	inactive_bs = inactive_border;

	// set the coords of the accessory
	r.left = x1; r.right = x2; r.top = y1; r.bottom = y2;

	// calculate max size required for the border
	int bs = max(grSizeOfBorder[active_border],grSizeOfBorder[inactive_border]);

	// and define our inset_r rectangle
	inset_r.left = r.left + bs; inset_r.right = r.right - bs;
	inset_r.top  = r.top + bs; inset_r.bottom = r.bottom - bs;

	// set default options
	selected_options = options;

	// set the button to be inactive by default
	isActive = false;

	// by default, all accessories are selectable
	isSelectable = true;

	// we are by default not selected!
	isSelected = false;

	// set the event to be returned
	returnEvent = retevent;

	// should we save what is underneath the accessory??
	if (selected_options&ac_SaveUnderAccessory)
		{
		// yes -> save image
		savedImage = (grImage *) malloc(grImageSize(r.left,r.top,r.right,r.bottom));
		if (savedImage!=NULL)
			{
			grHideMouse();
			grGetImage(r.left,r.top,r.right,r.bottom,savedImage);
			grShowMouse();
			}
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		DESTRUCTOR for Accessory
//	DESC:       This is the base destructor for _all_ Accessory's.
//
//	-------------------------------------------------------------------------
Accessory::~Accessory()
	{
	if (selected_options&ac_SaveUnderAccessory)
		if (savedImage!=NULL)
			{
			grHideMouse();
			grPutImage(r.left,r.top,r.right,r.bottom,savedImage);
			grShowMouse();
			free(savedImage);
			savedImage=NULL;
			}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::AdjustCoords()
//	DESC:       This function is called by event handlers (and accessories
//				that contain accessories ...) to offset the accessory
//				relative to the handler's (or accessory's) onscreen position
//
//	PARAMS:		dx	- +ve or -ve value to adjust coords (no bounds checking)
//				dy	- +ve or -ve value to adjust coords (no bounds checking)
//
//	-------------------------------------------------------------------------
void Accessory::AdjustCoords(int dx,int dy)
	{
	// adjust the outer rectangle
	r.left += dx;
	r.top += dy;
	r.right += dx;
	r.bottom += dy;
	// and also the inner one
	inset_r.left += dx;
	inset_r.top += dy;
	inset_r.right += dx;
	inset_r.bottom += dy;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::Select()
//	DESC:       This function selects the accessory.
//
//	-------------------------------------------------------------------------
void Accessory::Select()
	{
	// skip out if we're not selectable
	if (!isSelectable) return;

	if (!isSelected)
		{
		isSelected = true;
		grSetColor(LIGHTRED);
		grDrawRect(r.left-1,r.top-1,r.right+1,r.bottom+1);
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::DeSelect()
//	DESC:       This function deselects the accessory.
//
//	-------------------------------------------------------------------------
void Accessory::DeSelect()
	{
	if (!isSelectable) return;

	if (isSelected)
		{
		isSelected = false;
		grSetColor(LIGHTGRAY);
		grDrawRect(r.left-1,r.top-1,r.right+1,r.bottom+1);
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::Animate()
//	DESC:       This function is called automatically to animate the accessory
//
//	-------------------------------------------------------------------------
void Accessory::Animate()
	{
	// does nothing by default!!!
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::Draw()
//	DESC:       A simply draw function which fills the area underneath
//				the accessory and then draws a pretty border around this.
//
//	-------------------------------------------------------------------------
void Accessory::Draw()
	{
	if (isActive)
		grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);
	else
		grDrawWindow(r.left,r.top,r.right,r.bottom,fgcolor,inactive_bs);

	Refresh();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::Refresh()
//	DESC:       This function is used to refresh any information that may
//				have changed in an accessory thus minimising screen flicker.
//				This is automatically called by Accessory::Draw(), Select(),
//				DeSelect().
//
//	-------------------------------------------------------------------------
void Accessory::Refresh()
	{
	// does nothing by default!!
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Accessory::HandleEvent()
//	DESC:		This is the event-interpretation function for all Accessories.
//				Any event that is meant to go to an accessory arrives & is
//				interpreted here.
//
//	-------------------------------------------------------------------------
void Accessory::HandleEvent(Event &e)
	{
	// does nothing by default!!
	}


