//	-------------------------------------------------------------------------
//
//	DOSFILE.CPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file implements the DOSFile object which is used to
//					access a standard dos file (binary or text of course).
//
//	-------------------------------------------------------------------------

#include "dosfile.hpp"
#include "graphics.hpp"

#include "conio.h"
#include "stdio.h"
#include "fcntl.h"
#include "dos.h"
#include "sys\stat.h"
#include "errno.h"

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for DOSFile
//	DESC:       constructs an object using an external DirString specification
//
//	-------------------------------------------------------------------------
DOSFile::DOSFile(DirString* pathname,uint options)
	{
	fileName = pathname;
	createdStr = false;
	fileOptions = options;
	fileIsOpen = false;
	fileError = dosfile_NoError;

	Open();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for File
//	DESC:       constructs an object using an internal DirString specification
//
//	-------------------------------------------------------------------------
DOSFile::DOSFile(const char* pathname,uint options)
	{
	fileName = new DirString(pathname);
	createdStr = true;
	fileOptions = options;
	fileIsOpen = false;
	fileError = dosfile_NoError;

	Open();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		DESTRUCTOR for File
//	DESC:		clears up the file on delete...
//
//	-------------------------------------------------------------------------
DOSFile::~DOSFile()
	{
	if (fileIsOpen) Close();
	if (createdStr) delete fileName;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		FileSize()
//	DESC:		returns the current size of the file.
//
//	-------------------------------------------------------------------------
long DOSFile::FileSize()
	{
	return filelength(fileHandle);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		FilePos()
//	DESC:		returns the current position of the file pointer.
//
//	-------------------------------------------------------------------------
long DOSFile::FilePos()
	{
	return tell(fileHandle);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Seek()
//	DESC:		Sets the file pointer to an absolute offset from a specified
//				starting position
//	PARAMS:		pos			- the offset to seek to
//				startpos	- one of dosfile_SEEK_FROM_TOP
//                                   dosfile_SEEK_FROM_CURRENT
//									 dosfile_SEEK_FROM_END
//
//	-------------------------------------------------------------------------
int DOSFile::Seek(ulong pos, uint startpos)
	{
	// if the file is not open, return appropriate error...
	if (!fileIsOpen) return fileError = dosfile_FileNotOpen;

	// seek to the appropriate spot in the file
	long err = lseek(fileHandle,pos,startpos);

	// otherwise all is okay
	if ( err == -1 )
		return fileError = dosfile_SeekError;
	else
		return fileError = dosfile_NoError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Open()
//	DESC:       attempts to open the file specified when constructed
//
//	-------------------------------------------------------------------------
int DOSFile::Open()
	{
	if (fileIsOpen) return fileError = dosfile_FileAlreadyOpen;
	uint opt = O_BINARY;

	if (fileOptions&dosfile_READWRITE) opt |= O_RDWR;
	else
		{
		if (fileOptions&dosfile_READONLY) opt |= O_RDONLY;
		if (fileOptions&dosfile_WRITEONLY) opt |= O_WRONLY;
		}
	if (fileOptions&dosfile_APPEND) opt |= O_APPEND;
	if (fileOptions&dosfile_CREATE) opt |= O_TRUNC | O_CREAT;

	// O_CREAT -> open if !exists
	// O_TRUNC -> if exists -> truncate to 0 bytes

	if ((fileHandle = open(fileName->Contents(), opt, S_IREAD | S_IWRITE )) == -1)
		{
		switch (errno)
			{
			case ENOENT:	DBG(470,"DOSFILE::OPEN ERROR: No such file or directory"); break;
			case EMFILE:	DBG(470,"DOSFILE::OPEN ERROR: Too many open files"); break;
			case EACCES: 	DBG(470,"DOSFILE::OPEN ERROR: Permission denied"); break;
//			case EINVACC:	DBG(470,"DOSFILE::OPEN ERROR: Invalid access code"); break;
			}
		close(fileHandle);
		return fileError = dosfile_FileNotFound;
		}

	fileIsOpen = true;
	return fileError = dosfile_NoError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Close()
//	DESC:       closes an the file opened with Open()
//
//	-------------------------------------------------------------------------
int DOSFile::Close()
	{
	if (!fileIsOpen) return fileError = dosfile_FileAlreadyClosed;
	close(fileHandle);
	fileIsOpen = false;
	return fileError = dosfile_NoError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Read()
//	DESC:       reads 'size' bytes and places them in 'data'
//
//	-------------------------------------------------------------------------
int DOSFile::Read(char* data, long size)
	{
	// skip out if we're write-only
	if (fileOptions&dosfile_WRITEONLY) return fileError = dosfile_FileIsWriteOnly;

	// skip out if we're at eof
	if (Eof()) return fileError = dosfile_ReadError;

	if (size>64000L) DBG(450,"ABOUT TO READ %ld BYTES.",size);
	// if the record is less than read() maximum size,
	// read the record from the file
	int err = 0;

	if (size<=64000L) err = read(fileHandle,data,size);
	else
		// otherwise break it up into 64K chunks and read them instead
		{
		ulong siz = size;
		ulong iter = 0;
		do
			{
//			DBG(460,"YET TO READ %ld BYTES    ",siz);
			if (siz>64000L)
				{
				err = read(fileHandle,&data[iter*64000L],64000L);
				siz -= 64000L;
				}
			else
				{
				err = read(fileHandle,&data[iter*64000L],siz);
				siz = 0;
				}

			iter ++;
			if (err==-1) DBG(470,"---> ERROR: DOSFILE::READ returns %d ",err);
			}
		while ((siz!=0)&&(err!=-1));
		}

	uint width = (uint) data[8];
	uint height = (uint) data[10];

	// and return any errors
	if (err==-1)
		return fileError = dosfile_ReadError;
	else
		{
		return fileError = dosfile_NoError;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Write()
//	DESC:       writes 'size' bytes starting at 'data'
//
//	-------------------------------------------------------------------------
int DOSFile::Write(char* data, long size)
	{
	// skip out if we're read-only
	if (fileOptions&dosfile_READONLY) return fileError = dosfile_FileIsReadOnly;

	// if the record is less than write() maximum size,
	// write the record to the file
	if (size>64000L) DBG(450,"ABOUT TO WRITE %ld BYTES.",size);
	int err = 0;
	if (size<=64000L) err = write(fileHandle,data,size);
	else
		// otherwise break it up into 64K chunks and write them instead
		{
		ulong siz = size;
		ulong iter = 0;
		do
			{
			DBG(460,"YET TO WRITE %ld BYTES   ",siz);
			delay(1000);
			if (siz>64000L)
				{
				err = write(fileHandle,&data[iter*64000L],64000L);
				siz -= 64000L;
				}
			else
				{
				err = write(fileHandle,&data[iter*64000L],siz);
				siz = 0;
				}

			iter ++;
			if (err==-1) DBG(470,"---> ERROR: DOSFILE::WRITE returns %d",err);
			}
		while ((siz!=0)&&(err!=-1));
		}

	if (err==-1)
		return fileError = dosfile_WriteError;
	else
		return fileError = dosfile_NoError;
	}


