//	-------------------------------------------------------------------------
//
//	EVENTMGR.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the EVENT MANAGER.
//					The EVENTMGR maintains internally an EVENT QUEUE, which
//					can be used through the calls GenEvent() & ModEvent()
//
//	-------------------------------------------------------------------------

#include "eventmgr.hpp"
#include "mouse.hpp"
#include "keyboard.hpp"

#include "dos.h"
#include "mem.h"

// the GLOBAL declaration for our EVENTMGR!!!

EventMgr* EVENTMGR = NULL;

// a GLOBAL flag to tell whether or not the EVENTMGR is installed
bool	eventmgr_INSTALLED = false;


//	-------------------------------------------------------------------------
//
//  FUNC:		The CONSTRUCTOR
//	DESC:		Initializes the event manager, setting no event handlers,etc.
//
//	-------------------------------------------------------------------------

EventMgr::EventMgr()
	{
	numEventHandlers = 0;
	numEventsPending = 0;

	nextPending = 0;
	lastPending = 0;

	isActive = true;
	eventIsPending = false;
	eventQueueIsFull = false;
	eventHandlersAtMax = false;

	eventHandlersDisabled = false;

	// note: the first event handler is assumed to respond to keyboard input
	// 		 always!!! (May be a background handler)
	currentHandler = 0;
	currentZFactor = 1;	// ignore background tasks
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		The DESTRUCTOR
//	DESC:		Terminates the event manager, removing all event handlers
//
//	-------------------------------------------------------------------------
EventMgr::~EventMgr()
	{
	isActive = false;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		SelectNextHandler()
//	DESC:		Selects the next available handler (which is responding to
//				keyboard input) to be the selected keyboard input handler!
//
//	NOTE:		This function is meant to be used internally and _should_
//				only be used by system library programmers!
//
//	-------------------------------------------------------------------------
void EventMgr::SelectNextHandler()
	{
	// skip out if none or one event handlers
	if (numEventHandlers<2) return;

	// as we cannot select other handlers from dialog boxes...
	if (handler[currentHandler]->isDialogBox)
		{
		handler[currentHandler]->SelectNextAccessory();
		return;
		}

	// if the handler has no accessories or is at the last selected accessory
	if ((!handler[currentHandler]->isResponding)||
		(handler[currentHandler]->numAccessories==0)||
		(handler[currentHandler]->accessorySelected==handler[currentHandler]->numAccessories-1))
		{
		// deselect the current event handler
		handler[currentHandler]->DeSelect();

		// save the current handler
		int current = currentHandler;

		// and do at least once
		bool quit = false;
		do
			{
			// increment the current handler to the next
			current++;
			// wrap if out of bounds
			if (current>=numEventHandlers) current = 0;

			// quit out if we find a new handler or we're back at the start
			quit = (((handler[current]->isResponding)&&(handler[current]->isOnScreen))||
					 (current==currentHandler));
//			quit = ((handler[current]->isResponding)||
//					(current==currentHandler));
			}
		while (!quit);

		// set the new keyboard input handler
		currentHandler = current;

		// select the new event handler
		handler[currentHandler]->Select(evh_FIRST_ACCESSORY);
		}
	else
		// select the next accessory in the handler
		handler[currentHandler]->SelectNextAccessory();
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		SelectPrevHandler()
//	DESC:		Selects the previously available handler (which is responding
//				to keyboard input) to be the selected keyboard input handler!
//
//	NOTE:		This function is meant to be used internally and _should_
//				only be used by system library programmers!
//
//	-------------------------------------------------------------------------
void EventMgr::SelectPrevHandler()
	{
	// skip out if no event handlers
	if (numEventHandlers<2) return;

	// as we cannot select other handlers from dialog boxes...
	if (handler[currentHandler]->isDialogBox)
		{
		handler[currentHandler]->SelectPrevAccessory();
		return;
		}

	// if the handler has no accessories or is at the first selected accessory
	if ((!handler[currentHandler]->isResponding)||
		(handler[currentHandler]->numAccessories==0)||
		(handler[currentHandler]->accessorySelected==0))
		{
		// deselect the current event handler
		handler[currentHandler]->DeSelect();

		// save the current handler
		int current = currentHandler;

		// and do at least once
		bool quit = false;
		do
			{
			// decrement the current handler to the one before
			current--;
			// wrap if out of bounds
			if (current<0) current = numEventHandlers-1;

			// quit out if we find a new handler or we're back at the start
			quit = (((handler[current]->isResponding)&&(handler[current]->isOnScreen))||
					(current==currentHandler));
			}
		while (!quit);

		// set the new keyboard input handler
		currentHandler = current;

		// and select our new event handler
		handler[currentHandler]->Select(evh_LAST_ACCESSORY);
		}
	else
		// select the previous accessory in the handler
		handler[currentHandler]->SelectPrevAccessory();
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		SelectEventHandler( uint hand )
//	DESC:		Selects a specified event handler (which is responding to
//				mouse & keyboard input) as the current handler.
//
//	NOTES:		This function does not DeSelect() the current event handler!!
//
//				This function is meant to be used internally and _should_
//				only be used by system library programmers!
//
//	-------------------------------------------------------------------------
void EventMgr::SelectEventHandler( uint hand )
	{
	// skip out if dummy values
	if (hand>=numEventHandlers) return;

	// if its responding
	if ((handler[hand]->isResponding)&&
		// and its currently onscreen
		(handler[hand]->isOnScreen))
			{
			// select the new handler
			currentHandler = hand;
			handler[currentHandler]->Select( evh_CURRENT_ACCESSORY );
			}
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		DeSelectEventHandler( uint hand )
//	DESC:		DeSelects a specified event handler (which is responding to
//				mouse & keyboard input) so its not the current handler.
//
//	NOTES:		This function does not Select() the previously selected handler!!
//
//				This function is meant to be used internally and _should_
//				only be used by system library programmers!
//
//	-------------------------------------------------------------------------
void EventMgr::DeSelectEventHandler( uint hand )
	{
	// skip out if dummy values
	if (hand>=numEventHandlers) return;

	// if its responding
	if ((handler[hand]->isResponding)&&
		// and its currently onscreen
		(handler[hand]->isOnScreen))
			{
			// select the new handler
			handler[currentHandler]->DeSelect();
			}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		WhichHandlerSelected()
//	DESC:		Calculates which event handler was selected based upon a mouse
//				location point.
//
//	PARAMS:		ms_point	- the point to check
//								eg. int which = WhichHandlerSelected(e.mouse.where);
//
//	RETURNS		>=0 - the number of the handler selected
//				 -1 - no event handler was selected
//
//	-------------------------------------------------------------------------
int EventMgr::WhichHandlerSelected(grPoint& ms_point)
	{
	// if there are no accessories, return -1
	if (numEventHandlers==0) return -1;

	// by default we assume no handler selected
	int accessorynum = -1;
	int zfact = 0;

	// scan through each accessory
	for (int i=0;i<numEventHandlers;i++)
		// if the point is inside the bounds of the handler
		if ((ms_point.x>=handler[i]->r.left)&&(ms_point.x<=handler[i]->r.right)&&
			(ms_point.y>=handler[i]->r.top)&&(ms_point.y<=handler[i]->r.bottom)&&
			// and its on screen
			(handler[i]->isOnScreen)&&
			// and its responding
			(handler[i]->isResponding))
			{
			// if the handler's zfactor is the highest
			if (handler[i]->zfactor>=zfact)
				{
				// then thats the handler on top
				accessorynum = i;
				zfact = handler[i]->zfactor;
				}
			}

	// if the zfactor of the handler found was less than the current
	// zfactor, we ignore the result
	if (zfact<currentZFactor) accessorynum = -1;

	// return the result of our checking
	return accessorynum;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		DrawALLHandlers()
//	DESC:		Scans through the event handler list and tells all onscreen
//				handlers to Draw()
//
//	-------------------------------------------------------------------------
void EventMgr::DrawALLHandlers()
	{
	// scan through all of the handlers
	for (int i=0;i<numEventHandlers;i++)
		// and if they're on screen
		if (handler[i]->isOnScreen)
			// draw them
			handler[i]->Draw();
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		EventInQueue()
//	DESC:		scans the event queue, returning true if the specified event
//				was found in the queue
//
//	PARAMS:		what 	- the type of event to scan for
//				e 		- the specific event to scan for
//
//	-------------------------------------------------------------------------
bool EventMgr::EventInQueue(uint what,void* e)
	{
	// scan through the number of events pending
	for (int i=0;i<numEventsPending;i++)
		{
		// figure out the appropriate index into the queue
		int index = i + nextPending;
		// wrapping iff required
		if (index>EVENTMGR_DEFAULT_QUEUE_SIZE)
			index -= EVENTMGR_DEFAULT_QUEUE_SIZE;

		// if the pending event is of the appropriate type
		if (queue[index].what == what)
			// compare the events
			switch (what)
				{
				case	ev_timer:		if (memcmp(e,&queue[index].timer,sizeof(TimerEvent))==0)
											return true;
										break;
				case	ev_mouse:		if (memcmp(e,&queue[index].mouse,sizeof(MouseEvent))==0)
											return true;
										break;
				case	ev_keyboard:	if (memcmp(e,&queue[index].keyboard,sizeof(KeyboardEvent))==0)
											return true;
										break;
				case	ev_command:		if (memcmp(e,&queue[index].command,sizeof(CommandEvent))==0)
											return true;
										break;
				case	ev_broadcast:	if (memcmp(e,&queue[index].broadcast,sizeof(BroadcastEvent))==0)
											return true;
										break;
				}
		}

	// we didn't find it, return false...
	return false;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		GenEvent()
//	DESC:		Generate an event, placing it at the end of the Event Queue.
//
//	PARAMS:		where 	- the event handler where the event is going
//				what	- the type of event
//				data	- any relevant data accompanying the event
//
//	RETURNS:	true 	- if event was generated correctly.
//
//	-------------------------------------------------------------------------
//$$$ uint last_what;
bool EventMgr::GenEvent(uint where,uint what,void* data)
	{
	// if the event queue is full return false
	if (eventQueueIsFull) return false;

	// place appropriate bits in queue
	queue[lastPending].where = where;
	queue[lastPending].what  = what;
	// the data is defined by the type of event so....
	switch (what)
		{
		case ev_mouse:   	memcpy(&queue[lastPending].mouse,data,sizeof(MouseEvent)); 		break;
		case ev_keyboard:	memcpy(&queue[lastPending].keyboard,data,sizeof(KeyboardEvent)); 	break;
		case ev_timer:		memcpy(&queue[lastPending].timer,data,sizeof(TimerEvent)); 		break;
		case ev_command:	memcpy(&queue[lastPending].command,data,sizeof(CommandEvent)); 	break;
		case ev_broadcast:	memcpy(&queue[lastPending].broadcast,data,sizeof(BroadcastEvent));break;
		}

//$$$ grSetColor(BLACK);
//$$$ grDrawString(0,300,"Generated event %d -- %d",last_what,queue[lastPending].keyboard.event);
//$$$ last_what = what;
//$$$ grSetColor(LIGHTBLUE);
//$$$ grDrawString(0,300,"Generated event %d -- %d",last_what,queue[lastPending].keyboard.event);

	// figure out the next available slot
	lastPending++;
	if (lastPending>=EVENTMGR_DEFAULT_QUEUE_SIZE) lastPending = 0;

	// the event queue is full when the first == last
	eventQueueIsFull = (nextPending==lastPending);

	// tell the manager that an event is pending
	eventIsPending = true;

	// increment our events pending counter
	if (numEventsPending<EVENTMGR_DEFAULT_QUEUE_SIZE) numEventsPending++;

	// event was created correctly
	return true;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		InsEvent()
//	DESC:		Generate an event, placing it at the start of the Event Queue.
//
//	PARAMS:		where 	- the event handler where the event is going
//				what	- the type of event
//				data	- any relevant data accompanying the event
//
//	RETURNS:	true 	- if event was generated correctly.
//
//	-------------------------------------------------------------------------
//$$$ uint last_what;
bool EventMgr::InsEvent(uint where,uint what,void* data)
	{
	// if the event queue is full return false
	if (eventQueueIsFull) return false;

	// figure out the next available slot
	nextPending--;
	if (nextPending<0) nextPending = EVENTMGR_DEFAULT_QUEUE_SIZE - 1;

	// place appropriate bits in queue
	queue[nextPending].where = where;
	queue[nextPending].what  = what;
	// the data is defined by the type of event so....
	switch (what)
		{
		case ev_mouse:   	memcpy(&queue[nextPending].mouse,data,sizeof(MouseEvent)); 		break;
		case ev_keyboard:	memcpy(&queue[nextPending].keyboard,data,sizeof(KeyboardEvent)); 	break;
		case ev_timer:		memcpy(&queue[nextPending].timer,data,sizeof(TimerEvent)); 		break;
		case ev_command:	memcpy(&queue[nextPending].command,data,sizeof(CommandEvent)); 	break;
		case ev_broadcast:	memcpy(&queue[nextPending].broadcast,data,sizeof(BroadcastEvent));break;
		}

//$$$ grSetColor(BLACK);
//$$$ grDrawString(0,300,"Generated event %d -- %d",last_what,queue[lastPending].keyboard.event);
//$$$ last_what = what;
//$$$ grSetColor(LIGHTBLUE);
//$$$ grDrawString(0,300,"Generated event %d -- %d",last_what,queue[lastPending].keyboard.event);

	// the event queue is full when the first == last
	eventQueueIsFull = (nextPending==lastPending);

	// tell the manager that an event is pending
	eventIsPending = true;

	// increment our events pending counter
	if (numEventsPending<EVENTMGR_DEFAULT_QUEUE_SIZE) numEventsPending++;

	// event was created correctly
	return true;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		ModEvent()
//	DESC:		Modify a specified event (generally the current one).
//
//	PARAMS:     event	- the event to be modified
//				where 	- the event handler where the event is going
//				what	- the type of event
//				data	- any relevant data accompanying the event
//
//	RETURNS:	true if event was modified correctly.
//
//	-------------------------------------------------------------------------
void EventMgr::ModEvent(Event& event,uint where,uint what,void* data)
	{
	// replace appropriate bits in event
	event.where = where;
	event.what  = what;
	// the data is defined by the type of event so....
	switch (what)
		{
		case ev_mouse:   	memcpy(&event.mouse,data,sizeof(MouseEvent));		break;
		case ev_keyboard:	memcpy(&event.keyboard,data,sizeof(KeyboardEvent));	break;
		case ev_timer:		memcpy(&event.timer,data,sizeof(TimerEvent));		break;
		case ev_command:	memcpy(&event.command,data,sizeof(CommandEvent));	break;
		case ev_broadcast:	memcpy(&event.broadcast,data,sizeof(BroadcastEvent)); break;
		}
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		ClearEvent()
//	DESC:		Clears the current event, by simply skipping to the next
//				pending event
//
//	PARAMS		e	- the event to be cleared
//
//	-------------------------------------------------------------------------

void EventMgr::ClearEvent(Event &e)
	{
	// clear the event
	e.what = ev_nothing;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		ClearALLEvents()
//	DESC:		Clears all pending events by resetting the queue.
//
//	-------------------------------------------------------------------------

void EventMgr::ClearALLEvents()
	{
	// clear all pending events
	nextPending = 0;
	lastPending = 0;
	eventIsPending = false;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		AddEventHandler()
//	DESC:		Adds an event handler to the (internal) list of event handlers
//
//	PARAMS:		eventhandler	- the result of a new() call.
//						  			eg. MenuWindow* menu =
//										EVENTMGR->AddEventHandler(new(MenuWindow));
//
//	RETURNS:	a pointer to the resulting event handler
//
//	-------------------------------------------------------------------------
EventHandler* EventMgr::AddEventHandler(EventHandler* eventhandler)
	{
	// skip out if new didn't work!
	if (eventhandler==NULL) return NULL;

	// skip out upon reaching max event handlers
	if (eventHandlersAtMax)
		{
		// remove floating copy of the eventhandler
		delete eventhandler;
		// and return a null
		return NULL;
		}

	// set the event handler
	handler[numEventHandlers] = eventhandler;

	// increment the number of event handlers
	numEventHandlers++;

	// and check to see if we are at max...
	if (numEventHandlers==EVENTMGR_MAX_EVENT_HANDLERS-1) eventHandlersAtMax = true;

	// and return the pointer for later access
	return eventhandler;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		DelEventHandler()
//	DESC:		Deletes the most recently Add'ed event handler.
//
//	-------------------------------------------------------------------------
void EventMgr::DelEventHandler()
	{
	// skip out if there are no eventhandlers
	if (numEventHandlers==0) return;

	// decrement the number of event handlers
	numEventHandlers--;

	// and delete the event handler
	delete handler[numEventHandlers]; handler[numEventHandlers] = NULL;
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		DelALLEventHandlers()
//	DESC:		Deletes all of the event handlers...
//
//	NOTE:		do not use outside of PROGRAM.CPP
//
//	-------------------------------------------------------------------------
void EventMgr::DelALLEventHandlers()
	{
	// delete all of the event handlers using DelEventHandler
	while (numEventHandlers>0)
		DelEventHandler();
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		HandleEvent()
//	DESC:		Handles the next pending event.
//
//				If the pending event's _where_ field is set to an event
//				handler then that event handler is passed the event immediately
//
//				If the pending event's _where_ field == ALLHANDLERS then all
//				EventHandlers are called in order of allocation till one of
//				the handlers handles the event!
//
//	-------------------------------------------------------------------------
//int xpos,ypos;
//int lastkey;
int tab_cnt = 0;
void EventMgr::HandleEvent(Event &e)
	{
	// create this here
	uint nexthandler = 0;
	int hand;

	// do the switch (in order of speed preference!!)
	switch (e.what)
		{
		// if its a command event, go to its destination (allowing chaining!)
		// exception: if e.command.event == cmdExitProgram, terminate!
		case	ev_command:		if (e.command.event==cmdExitProgram)
									{
									// finished executing event loop
									isExecuting = false;

									// so skip out
									return;
									}

								// skim through, chaining command events!
								while (e.what==ev_command)
									handler[e.where]->HandleEvent(e);
								break;

		// if its a timer or a broadcast event
		case	ev_timer:		;
		case	ev_broadcast:	// start at the first event handler
								nexthandler = 0;

								// loop through all the event handlers
								do
									{
									// sending the event to the next handler
									handler[nexthandler]->HandleEvent(e);

									// and increment
									nexthandler++;
									}
								while (nexthandler<numEventHandlers);

								// broadcasts & timers are once-only, so clear
								ClearEvent(e);
								break;

		// if its a mouse event, transfer the event to the appropriate handler
		case	ev_mouse:       // figure out which handler was selected
								hand = WhichHandlerSelected(e.mouse.where);

								// if a handler was selected
								if (hand>=0)
									{
									// if we have selected a different handler, tell that handler to deselect
									if (hand!=currentHandler)
										{
										handler[currentHandler]->DeSelect();
										handler[hand]->Select( evh_CURRENT_ACCESSORY );
										currentHandler = hand;
										}

									// and tell the handler to handle the event
									handler[currentHandler]->HandleEvent(e);
									}

								// if no handler has interpreted the info, clear it
								if (e.what==ev_mouse) ClearEvent(e);
								break;

		// if its a keyboard event, go straight to our currently selected
		// handler. Otherwise try all of the background handlers in turn!

		case	ev_keyboard:	// try the currently selected handler
								handler[currentHandler]->HandleEvent(e);

								// attempt the interface!!!!
//								if (e.what==ev_keyboard)
//									handler[1]->HandleEvent(e);

								// if the event wasn't handled, catch it now!
								if (e.what==ev_keyboard)
									{
									// SPECIAL CASE: CHECK FOR ALT_Q OR CTRL_Q
									//				 TO QUIT PROGRAM!!!
									switch (e.keyboard.event)
										{
										// handle the tab being released
										case TAB_DN	:   tab_cnt++;
														if (tab_cnt>300)
															{
															tab_cnt = 0;

															// if its shift-tab
															if (e.keyboard.modifiers&keyboard_SHIFT)
																SelectPrevHandler();
															else
																// its normal tab
																SelectNextHandler();
															}

														// clear the event
														ClearEvent(e);
														break;

										case	ESC_UP: if ((e.keyboard.modifiers&keyboard_ALT)||
															(e.keyboard.modifiers&keyboard_CTRL))
															// turn off Execute()!
															{
															isExecuting = false;
															return;
															}
										default:        // if it didn't use it, attempt background handlers
														// as long as its not TAB_UP
														if (e.keyboard.event==TAB_UP) break;

														// start at the first handler
														nexthandler = 0;

														// loop through all event handlers
														do
															{
															// if the handler hasn't been checked already
															if (nexthandler!=currentHandler)
																// if the next handler is responding
																if (handler[nexthandler]->isResponding)
																	handler[nexthandler]->HandleEvent(e);

															// if its still a keyboard event, try the next
															if (e.what==ev_keyboard) nexthandler++;
															else
																// skip out as it's a new event
																nexthandler=numEventHandlers;
															}
														while (nexthandler<numEventHandlers);
														break;
										}

									// clear event regardless as it wasn't handled
									if (e.what==ev_keyboard) ClearEvent(e);
									}
								break;

		// if its a non-event, skip out after clearing the event!
		default:				ClearEvent(e);
								break;
		}


	// if the event is now nothing, fix up the queue!
	if (e.what==ev_nothing)
		{
		// update the range of events that are pending
		nextPending++;
		if (nextPending==EVENTMGR_DEFAULT_QUEUE_SIZE) nextPending = 0;

		// and set the global pending flag
		eventIsPending = (nextPending!=lastPending);

		// event queue is no longer full
		eventQueueIsFull = false;

		// and decrement our number of events pending
		numEventsPending--;
		}
	}

//	-------------------------------------------------------------------------
//
//  FUNC:		Execute()
//	DESC:		Performs the main loop of the event-handling mechanism.
//				It continuously loops performing the following algorithm:
//
//				DO
//					IF (event_is_pending) THEN handle_the_pending_event
//					ELSE perform_an_idle_function
//              UNTIL end_of_program
//
//				In the current version of the library, PERFORM_AN_IDLE_FUNCTION
//				checks for mouse input!
//
//	-------------------------------------------------------------------------
void EventMgr::Execute()
	{
	// if there are no event handlers, skip out!!!
	if (numEventHandlers==0) return;

	// we are now executing the event loop
	isExecuting = true;

	uint iter = 0;

	// otherwise loop till we've recieved finished
	while (isExecuting)
		{
		// if there is an event pending
		if (eventIsPending)
			{
			// handle the next pending event
			HandleEvent(queue[nextPending]);
			}
		else
			{
			// there was no event pending
//			timer_GetEvent();
//			if (++iter>200)
				{
//				iter = 0;
				//Idle();
				// we poll the mouse & keyboard in this spare time!!!
				mouse_GetEvent();
				}
			keyboard_GetEvent();
			}
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		eventmgr_InstallEVENTMGR()
//	DESC:		creates our event manager & stores it as the global EVENTMGR
//
//	RETURNS:	true 	- if installed correctly
//				false	- error occurred during installation
//
//	-------------------------------------------------------------------------
bool eventmgr_InstallEVENTMGR()
	{
	// if we aren't already installed
	if (!eventmgr_INSTALLED)
		{
		// attempt to create our event manager
		EVENTMGR = new EventMgr();
		// if it was created okay
		if (EVENTMGR!=NULL)
			{
			// set flag
			eventmgr_INSTALLED = true;
			// and return true
			return true;
			}
		}

	// otherwise(s) return false
	return false;
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		eventmgr_RemoveEVENTMGR()
//	DESC:		deletes the EVENTMGR, thus closing down all eventhandlers.
//
//	-------------------------------------------------------------------------
void eventmgr_RemoveEVENTMGR()
	{
	// if its actually installed
	if (eventmgr_INSTALLED)
		{
		// delete the event manager
		delete EVENTMGR; EVENTMGR = NULL;

		// and set our flag to 'not installed'
		eventmgr_INSTALLED = false;
		}
	}
