//	-------------------------------------------------------------------------
//
//	FILES.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the File object which is used to
//					access a pseudo-random-access file of variable-size
//					records.
//
//	-------------------------------------------------------------------------

#include "files.hpp"

#include "fcntl.h"
#include "dos.h"
//#include "dir.h"
#include "sys\stat.h"
#include "errno.h"

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for File
//	DESC:       constructs an object using an external DirString specification
//
//	-------------------------------------------------------------------------
File::File(DirString* pathname,uint options)
	{
	FileName = pathname;
	created_str = false;
	tmpFileName = new DirString("TEMPORRY.TMP");
	file_options = options;
	FileIsOpen = false;
	fileError = file_NoError;
	record_pointer = -999;
	record_size = -999;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for File
//	DESC:       constructs an object using an internal DirString specification
//
//	-------------------------------------------------------------------------
File::File(const char* pathname,uint options)
	{
	FileName = new DirString(pathname);
	created_str = true;
	tmpFileName = new DirString("TEMPORRY.TMP");
	file_options = options;
	FileIsOpen = false;
	fileError = file_NoError;
	record_pointer = -999;
	record_size = -999;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		DESTRUCTOR for File
//	DESC:		clears up the file on delete...
//
//	-------------------------------------------------------------------------
File::~File()
	{
	if (FileIsOpen) Close();
	delete tmpFileName;
	if (created_str) delete FileName;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		SetFileOptions()
//	DESC:		Allows the file to change its name, or its mode of operation
//				or both...
//
//	-------------------------------------------------------------------------
void File::SetFileOptions(DirString* filename,uint options)
	{
	bool open = FileIsOpen;
	if (open) Close();
	FileName->Set(filename->Contents());
	file_options = options;
	if (open) Open();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Open()
//	DESC:       attempts to open the file specified when constructed
//
//	-------------------------------------------------------------------------
int File::Open()
	{
	if (FileIsOpen) return fileError = file_FileIsOpen;
	uint opt = O_BINARY;

	if (file_options&file_READWRITE) opt |= O_RDWR;
	else
		{
		if (file_options&file_READONLY) opt |= O_RDONLY;
		if (file_options&file_WRITEONLY) opt |= O_WRONLY;
		}
	if (file_options&file_APPEND) opt |= O_APPEND;
	if (file_options&file_CREATE) opt |= O_TRUNC | O_CREAT;

	// O_CREAT -> open if !exists
	// O_TRUNC -> if exists -> truncate to 0 bytes

	if ((filehandle = open(FileName->Contents(), opt, S_IREAD | S_IWRITE )) == -1)
		{
		switch (errno)
			{
			case ENOENT:	printf("\nNo such file or directory"); break;
			case EMFILE:	printf("\nToo many open files"); break;
			case EACCES: 	printf("\nPermission denied"); break;
//			case EINVACC:	printf("\nInvalid access code"); break;
			}
		close(filehandle);
		return fileError = file_FileNotFound;
		}

	FileIsOpen = true;
	return fileError = file_NoError;
	}

int File::Close()
	{
	if (!FileIsOpen) return fileError = file_FileIsClosed;
	close(filehandle);
	FileIsOpen = false;
	return fileError = file_NoError;
	}

int File::Delete(long recnum)
	{
	if (file_options&file_READONLY) return fileError = file_FileIsClosed;
	if (!FileIsOpen) return fileError = file_FileIsClosed;
	if (Seek(recnum)!=file_NoError) return fileError;

	File* tmpfile = new File(tmpFileName,file_READWRITE|file_CREATE);
	tmpfile->Open();

	long rec = 0;
	long siz = 0;
	char* tmpbuf = NULL;
	// copy all the records up to the one we are deleting
	while (rec<recnum)
		{
		siz = Recsize(rec);
		tmpbuf = (char*) malloc(siz);
		Read(rec,tmpbuf);
		tmpfile->Write(rec,tmpbuf,siz);
		free(tmpbuf);
		rec++;
		}

	// then copy all following records, skipping the deleted one
	while (!eof(filehandle))
		{
		siz = Recsize(rec+1);
		tmpbuf = (char*) malloc(siz);
		Read(rec+1,tmpbuf);
		tmpfile->Write(rec,tmpbuf,siz);
		free(tmpbuf);
		rec++;
		}

	// close the temporary file & our data file
	delete tmpfile;
	Close();

	// delete our original file & rename the temporary file as the data file
	remove(FileName->Contents());
	rename(tmpfile->FileName->Contents(),FileName->Contents());

	// open our file again
	Open();

	// should be okay....
	return fileError = file_NoError;
	}

int File::Insert(long recnum,long size)
	{
	if (file_options&file_READONLY) return fileError = file_FileIsClosed;
	if (!FileIsOpen) return fileError = file_FileIsClosed;
	if (Seek(recnum)!=file_NoError) return fileError;

	File* tmpfile = new File(tmpFileName,file_READWRITE|file_CREATE);
	tmpfile->Open();

	long rec = 0;
	long siz = 0;
	char* tmpbuf = NULL;
	// copy all the records up to the one we are deleting
	while (rec<recnum)
		{
		siz = Recsize(rec);
		tmpbuf = (char*) malloc(siz);
		Read(rec,tmpbuf);
		tmpfile->Write(rec,tmpbuf,siz);
		free(tmpbuf);
		rec++;
		}

	// insert a new record
	tmpbuf = (char*) malloc(size+1);
	memset(tmpbuf,32,size);
	tmpfile->Write(rec,tmpbuf,size+1);
	free(tmpbuf);

	// then copy all records following recnum
	while (!eof(filehandle))
		{
		siz = Recsize(rec);
		tmpbuf = (char*) malloc(siz);
		Read(rec,tmpbuf);
		tmpfile->Write(rec+1,tmpbuf,siz);
		free(tmpbuf);
		rec++;
		}

	// close the temporary file & our data file
	delete tmpfile;
	Close();

	// delete our original file & rename the temporary file as the data file
	remove(FileName->Contents());
	rename(tmpfile->FileName->Contents(),FileName->Contents());

	// open our file again
	Open();

	// should be okay....
	return fileError = file_NoError;
	}

int File::Seek(long recnum)
	{
	// if the file is not open, return appropriate error...
	if (!FileIsOpen) return fileError = file_FileNotOpen;

	// seek to top of file
	lseek(filehandle,0L,SEEK_SET);
	long rec = 0;	// record ptr
	long siz = 0;	// record size

	// scan through the file, reading the headers and skipping past...
	// until we reach the appropriate record or eof()

	int err;
	// if its the first record we're after, read the appropriate bits
	if (recnum==0)
		{
		err = read(filehandle,&siz,sizeof(long));
		// if an error occurs, skip out as we are at eof() most prob...
		if (err<=0) return fileError = file_RecordNotDefined;
		}
	else
		{
		// search through the file until we get to the right record
		bool searching = true;
		while (searching)
			{
			// seek the next record
			lseek(filehandle,siz,SEEK_CUR);

			// read the size of the next record
			err = read(filehandle,&siz,sizeof(long));

			// if we have found the right record, finished searching
			if (rec==recnum) searching = false;
			// we have hit the end of file, finished searching
			if (eof(filehandle)) searching = false;

			// and increment our temporary record pointer
			if (searching) rec++;
			}
		}

	// so set the file pointer to the size of the record.. etc.
	record_pointer = rec;
	record_size = siz;

	// if we're at the end of file, tell the thingy to write the record
	if (eof(filehandle)) return fileError = file_RecordNotDefined;
	// otherwise all is okay
	return fileError = file_NoError;
	}

long File::Recsize(long recnum)
	{
	// if we can't find the record return the last reported error
	if (Seek(recnum)!=file_NoError) return fileError;
	// otherwise return its size
	return record_size;
	}

int File::Read(long recnum, void* data)
	{
	if (file_options&file_WRITEONLY) return fileError = file_FileIsClosed;
	// if the record doesn't exist, return
	if (Seek(recnum)!=file_NoError) return fileError = file_InvalidRecord;

	// read the record from the file
	long err;
	err = read(filehandle,(char*) data,record_size);
	if (err<=0) return fileError = file_InvalidRecord;

	// and return all ok
	return fileError = file_NoError;
	}

int File::Write(long recnum, void* data, long size)
	{
	if (file_options&file_READONLY) return fileError = file_FileIsClosed;
	// if the record number specified exists...
	if (Seek(recnum)!=file_NoError)
		if (fileError!=file_RecordNotDefined) return fileError;

	bool write_header_info = (fileError == file_RecordNotDefined);
	// if we need to reorganize the file (to accomodate different record size)
	// then do so...
	if (record_size!=size)
		{
		Delete(recnum);
		Insert(recnum,size);
		write_header_info = true;
		}

	// if we need to write the header info, write it...
	if (write_header_info)
		write(filehandle,&size,sizeof(long));

	// write the data...
	write(filehandle,(char*)data,size);

	// all is ok...
	return fileError = file_NoError;
	}

/*
// DIRSTRING FUNCTIONS

void DirString::SetFileName(const char* st)
	{
	int start = ScanLeft(Length(),'\\')+1;
	Delete(start,Length()-start);
	Append(st);
	}

void DirString::SetDrive(const char* st)
	{
	int end = ScanRight(0,':') + 1;
	Delete(0,end);
	Insert(0,st);
	}

void DirString::SetPath(const char* st)
	{
//	DBG->Display("input string  == >%s<",st);
	int start = ScanRight(0,':') + 1;
//	if (start>Length()) start = 0;
	int end = ScanLeft(Length(),'\\');
//	if (end>Length()) end = Length();
//	DBG->Display("start, end    == %d,%d",start,end);

//	DBG->Display("start string  == >%s<",Contents());
//	DBG->Display("before delete == >%s<",Contents());
	if (start<end) Delete(start+1,end-start);
//	DBG->Display("after delete  == >%s<",Contents());

//	DBG->Display("before insert == >%s<",Contents());
//	DBG->Display("after insert  == >%s<",Contents());

//	DBG->Display("before insert == >%s<",Contents());
	// insert the new path string
	Insert(start,st);
//	DBG->Display("after insert  == >%s<",Contents());
	// if there is no '\' at the end of the path, put it in now!
	if (Contents()[2]!='\\') Insert(2,'\\');
//	if (st[strlen(st)]!='\\') Insert(start,'\\');

	// if there is no '\' at the start of the path, put it in now!
	// (except for when we are in the root directory!!!)
//	DBG->Display("before insert == >%s<",Contents());
//	if ((st[0]!='\\')&&(strlen(st)!=0)) Insert(start,'\\');
//	DBG->Display("output string == >%s<",Contents());
	}

void DirString::SetDirectory(const char* st)
	{
	int start = ScanLeft(Length(),'\\') + 1;

	// if there is no '\' at the end of the st, put it in
	if (st[strlen(st)]!='\\') Insert(start,'\\');

	Insert(start,st);
	}

String* DirString::GetFileName()
	{
	int end = Length();
	int start = ScanLeft(end,'\\')+1;
	String* tmpstr = SubStr(start,end);
	return tmpstr;
	}

String* DirString::GetDrive()
	{
	int end = ScanRight(0,':');
	String* tmp = SubStr(0,end);
	return tmp;
	}

String* DirString::GetPath()
	{
	int start = ScanRight(0,':') + 1;
	int end = ScanLeft(Length(),'\\');
	String* tmp = SubStr(start,end);
	return tmp;
	}

String* DirString::GetDirectory()
	{
	int end = ScanLeft(Length(),'\\') - 1;
	int start = ScanLeft(end,'\\') + 1;
	String* tmp = SubStr(start,end);
	return tmp;
	}

// other functions for use with files & directories

bool fileExists(DirString* pathname)
	{
	int handle = open(pathname->Contents(), O_RDWR, S_IREAD | S_IWRITE );
	if (handle == -1) return false;
	else { close(handle); return true; }
	}

int FilesInDir(uint attrib)
	{
	struct ffblk ffblk;
	int attr = 0;
	switch (attrib)
		{
		case	dir_DirectoriesOnly:	attr |= FA_DIREC; break;
		case 	dir_FilesOnly:			break;
		case	dir_AllFiles:			attr |= FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_LABEL | FA_DIREC | FA_ARCH;
		}

	int done;
	int cnt=0;
	done = findfirst("*.*",&ffblk,attr);
	cnt++;

	while (!done)
		{
		done = findnext(&ffblk);
		cnt++;
		}
	return cnt;
	}

void ReadDirectory(String* dirstr,uint attrib)
	{
	ffblk fb;
	int attr = 0;
	int done;

	switch (attrib)
		{
		case	dir_DirectoriesOnly:	attr |= FA_DIREC; break;
		case 	dir_FilesOnly:			attr |= 32; break;
		case	dir_AllFiles:			attr |= FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_LABEL | FA_DIREC | FA_ARCH;
		}

	dirstr->Set("");
	done = findfirst("*.*",&fb,attr);

	int cnt = 0;
	while (!done)
		{
		if (fb.ff_attrib==attr)   // NOTE:: BUGFIX... check attrib manually & findnext doesn't??
			{
//			DBG->Display("filename #%d == >%s<",cnt++,fb.ff_name);
			dirstr->Append(fb.ff_name);
			dirstr->Append("/");
			}
		done = findnext(&fb);
		}
	dirstr->Delete(dirstr->Length()-1,1);
//	if (heapcheck()==_HEAPCORRUPT) DBG->Display("HEAP IS CORRUPT! (ReadDirectory)");
	}

void GetDriveNames(String* diskstr)
	{
	int current_disk = getdisk();
	int num_drives = setdisk(current_disk);

	diskstr->Set("");
	for (int i=0;i<num_drives;i++)
		{
		diskstr->Append((char)65+i);
		diskstr->Append(":/");
		}
	diskstr->Delete(diskstr->Length(),1);
	}

void GetCurrentPath(String* pathstr)
	{
	// make blank string
	DirString* tmpstr = new DirString("");
	// add current path  "\CURRENT\DIRECTRY\"
	getcurdir(0,tmpstr->Contents());
	// and use this to modify pathstr
	pathstr->Set(tmpstr->Contents());
	// get rid of the temp string
	delete tmpstr;
	}
*/