//	-------------------------------------------------------------------------
//
//	GRAPHICS.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the base level graphics functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------

#ifndef __graphics_hpp
#define __graphics_hpp

#include "stddefns.hpp"

#include "string.h"	// temporary

// GUI-specific items

// standard palette used by graphics functions

#define	grBLACK			0
#define grBLUE			1
#define	grGREEN			2
#define grCYAN			3
#define	grRED			4
#define	grMAGENTA		5
#define	grBROWN			6
#define grLIGHTGRAY		7
#define grDARKGRAY		8
#define grLIGHTBLUE		9
#define grLIGHTGREEN	10
#define	grLIGHTCYAN		11
#define	grLIGHTRED		12
#define	grLIGHTMAGENTA	13
#define	grYELLOW		14
#define	grWHITE			15

extern uchar	gr_HIGHLIGHT;
extern uchar	gr_LOWLIGHT;
extern uchar	gr_BACKGROUND;

extern uchar 	FGCOLOR;
extern uchar	BGCOLOR;
extern uchar	LIGHTCOLOR;
extern uchar	DARKCOLOR;

// the format of the generic "point"
struct grPoint
	{
	uint x;
	uint y;
	};

// the format of the generic "rectangle"
struct grRect
	{
	uint left;
	uint top;
	uint right;
	uint bottom;
	};

// the image header used by the graphics library

struct grImageHeader
	{
	uint	width;				// width of image in pixels
	uint	height;				// height of image in pixels
	};

// and the size of the header as a constant!!

#define		grImage_HEADER_SIZE			4L

// the format of our generic "image" (or bitmap)
struct grImage
	{
	grImageHeader hdr;
	uchar data[];
	};

struct grPaletteHeader
	{
	uchar numentries;
	};

struct grPalette
	{
	grPaletteHeader hdr;
	uchar data[];
	};

// our virtual buffers are in fact grImages!!!

#define grVirtualBuffer		grImage

// as are our fonts (temporarily)

#define grFont				grImage

// font engine error messages returned from grFont... functions

#define		font_NoFontsLoaded		-1
#define		font_OutOfTags			-2
#define		font_OutOfBounds		-3
#define		font_OutOfMemory		-4

// simple 256 color palette structure used by Palette functions

struct grFullPalette
	{
	uchar numEntries;
	char data[];
	};

//	the sprite header

struct grSpriteHeader
	{
	int		number_frames;		// total number of frames
	int		width,				// width of image in frames
			height;				// height of image in frames
	};

// the sprite format

struct grSprite
	{
	grSpriteHeader	hdr;			// contains info for memory, number of frames and sizes
	grImage**		frame;			// list of images to make animation

	// this has to be initialised when sprite is loaded
	int		x,
			y,				// current x y position
			oldx,
			oldy;			// last x y position
	char	current_frame;	// current frame we are showing
	char	state;			// turning, dead, stopped, going etc
	grImage*background;		// background under sprite
	};

// some different border styles used in grDrawWindow(), etc.

#define bs_NONE			0
#define bs_FRAMED		1
#define bs_EDGED		2
#define bs_RAISED		3
#define bs_LOWERED		4
#define bs_DBLRAISED	5
#define bs_DBLLOWERED   6
#define bs_OUTLINE		7
#define bs_QUADRAISED	8
#define bs_QUADLOWERED	9

// the sizes of these borders (both defines & as array for easy access!!!)

#define bSize_NONE			0
#define bSize_FRAMED		6
#define bSize_EDGED			6
#define bSize_RAISED		1
#define bSize_LOWERED		1
#define bSize_DBLRAISED		2
#define bSize_DBLLOWERED	2
#define bSize_OUTLINE		1
#define	bSize_QUADRAISED	8
#define bSize_QUADLOWERED	8

// array of border sizes. 	eg. bordersize = grSizeOfBorder[DBLRAISED];

const int grSizeOfBorder[] = { bSize_NONE, bSize_FRAMED, bSize_EDGED,
							   bSize_RAISED, bSize_LOWERED, bSize_DBLRAISED,
							   bSize_DBLLOWERED, bSize_OUTLINE,
							   bSize_QUADRAISED, bSize_QUADLOWERED };

// GLOBAL variables for instant access to display information

extern uint 	grMaxx;			// the maximum x coordinate
extern uint 	grMaxy;			// the maximum y coordinate
extern uchar    grDrawColor;	// current default draw color
extern bool 	grMouseIsHidden;// whether or not the mouse is currently hidden

extern grImage BLANK_CURSOR;	// a blank mouse cursor
extern grImage ARROW_CURSOR;	// an arrow mouse cursor


// clipping macro for ease-of-use

#define CLIP(X,Y,X1,Y1,X2,Y2)		if ((X)<(X1)) (X)=(X1); \
									else if ((X)>(X2)) (X)=(X2); \
									if ((Y)<(Y1)) (Y)=(Y1); \
									else if ((Y)>(Y2)) (Y)=(Y2)

// different graphics modes available

#define gr640x480x256		0x0101

// --------------------------------------------------------------------------
//
//	VESA DRIVER FUNCTIONS
//
// --------------------------------------------------------------------------

// attempts to initialize vesa... should only ever need to call this once.
// returns false if cannot initialize vesa mode
bool grInitVesa(uint mode);

// terminates VESA mode
void grExitVesa();

// clear video page (to BLACK by default)
void grClearVideo(uchar color = grBLACK);

// --------------------------------------------------------------------------
//
//	VIRTUAL DISPLAY FUNCTIONS
//
// --------------------------------------------------------------------------

// create a virtual display buffer to store an image
grVirtualBuffer* grCreateVirtual(uint x1,uint y1,uint x2,uint y2);

// destroy a virtual display buffer
void grDestroyVirtual(grVirtualBuffer* virt);

// clear a virtual display (to BLACK by default)
void grClearVirtual(grVirtualBuffer* virt,uchar color = grBLACK);

// bitblit a virtual buffer onto the screen
#define grBlitVirtual(V,X1,Y1,X2,Y2) grPutImage((X1),(Y1),(X2),(Y2),(grImage*) (V))

// scroll a virtual buffer by dx,dy pixels
void grScrollVirtual(grVirtualBuffer* virt,int dx,int dy);

// --- note: the following functions draw to a virtual buffer with clipping
//			 and using (by default) the current draw color.

// draw a pixel into a virtual buffer at x,y coords
void grPutPixelVirtual(grVirtualBuffer* virt,uint x,uint y,uchar color = grDrawColor);

// get a pixel from a virtual buffer at x,y coords
uchar grGetPixelVirtual(grVirtualBuffer* virt,uint x,uint y);

// put an image into a virtual buffer starting at x1,y1
void grPutImageVirtual(grVirtualBuffer* virt,uint x1,uint y1,uint x2,uint y2,grImage* image);

// overlay an image into a virtual buffer starting at x1,y1
void grOverlayImageVirtual(grVirtualBuffer* virt,uint x1,uint y1,uint x2,uint y2,grImage* image);

// draw a line in a virtual buffer from x1,y1 to x2,y2
void grDrawLineVirtual(grVirtualBuffer* virt,uint x1, uint y1, uint x2, uint y2, uchar color = grDrawColor);

// draw a vertical line in a virtual buffer from x,y1 to x,y2
void grDrawVLineVirtual(grVirtualBuffer* virt,uint x,uint y1,uint y2,uchar color = grDrawColor);

// draw a horizontal line in a virtual buffer from x1,y to x2,y
void grDrawHLineVirtual(grVirtualBuffer* virt,uint y,uint x1,uint x2,uchar color = grDrawColor);

// draw a rectangle in a virtual buffer from x1,y1 to x2,y2
void grDrawRectVirtual(grVirtualBuffer* virt,uint x1, uint y1, uint x2, uint y2, uchar color = grDrawColor);

// draw a filled rectangle in a virtual buffer from x1,y1 to x2,y2
void grDrawFilledRectVirtual(grVirtualBuffer* virt,uint x1,uint y1,uint x2,uint y2, uchar color = grDrawColor);

// draw a string in a virtual buffer at the specified coordinates using variable args
void grDrawStringVirtual(grVirtualBuffer* virt,int x,int y,char* fmt,...);

// --------------------------------------------------------------------------
//
//	DIRECT-TO-VIDEO FUNCTIONS
//
// --------------------------------------------------------------------------

// set the default drawing color
void grSetColor(uchar color)	{ grDrawColor = color; }

// get the default drawing color
uchar grGetColor()				{ return grDrawColor; }

// --- note: the following functions do not clip the output..... beware.

// draw a pixel on the screen at x,y coords in specified color
void grPutPixel(uint x, uint y, uchar color = grDrawColor);

// get a pixel from the x,y coords on the screen
uchar grGetPixel(uint x, uint y);

// put an image onto the screen at x1,y1 to x2,y2
void grPutImage(uint x1,uint y1,uint x2,uint y2,grImage* image);

// overlay an image onto the screen at x1,y1 to x2,y2
void grOverlayImage(uint x1,uint y1,uint x2,uint y2,grImage* image);

// get an image off the screen into some buffer
void grGetImage(uint x1,uint y1,uint x2,uint y2,grImage* image);

// returns the size of the image specified by x1,y1,x2,y2
ulong grImageSize(uint x1,uint y1,uint x2,uint y2);

// draw a line from x1,y1 to x2,y2 in specified color
void grDrawLine(uint x1, uint y1, uint x2, uint y2, uchar color = grDrawColor);

// draw a vertical line from x,y1 to x,y2
void grDrawVLine(uint x,uint y1,uint y2,uchar color = grDrawColor);

// draw a horizontal line from x1,y to x2,y
void grDrawHLine(uint y,uint x1,uint x2,uchar color = grDrawColor);

// draw a rectangle from x1,y1 to x2,y2 in specified color
void grDrawRect(uint x1, uint y1, uint x2, uint y2, uchar color = grDrawColor);

// draw a filled rectangle from x1,y1 to x2,y2 in specified color
void grDrawFilledRect(uint x1,uint y1,uint x2,uint y2, uchar color = grDrawColor);

// draw a string at the specified coordinates using variable args
void grDrawString(int x,int y,char *fmt, ...);

// draw a window from x1,y1 to x2,y2 using border style 'bs' and a background
// color. NOTE: If background color==0 (BLACK) we assume no floodfill!!!
void grDrawWindow(uint x1,uint y1,uint x2,uint y2,uchar bg,uint bs);

// draws a vertical bar from xx,y1 to xx,y2 using border style 'bs'
void grDrawVBar(uint xx,uint y1,uint y2,uint bs);

// draws a horizontal bar from x1,yy to x2,yy using border style 'bs'
void grDrawHBar(uint yy,uint x1,uint x2,uint bs);

// --------------------------------------------------------------------------
//
//	FONT FUNCTIONS
//
// --------------------------------------------------------------------------
//	To implement a font
int		grInitFont(char* fontname);
int		grUseFont(int whichFont);
int		grExitFont(int whichFont);
int		grExitALLFonts();

// --------------------------------------------------------------------------
//
//	SPRITE FUNCTIONS
//
// --------------------------------------------------------------------------
grSprite*	grCreateSprite( char number_frames, int width, int height );
void 	grDestroySprite( grSprite* spr );
void	grEraseSprite( grSprite* spr );
void	grMoveSprite( grSprite* spr, int dx, int dy );
void	grSaveUnderSprite( grSprite* spr );
void	grDrawSprite( grSprite* spr );

// --------------------------------------------------------------------------
//
//	PALETTE FUNCTIONS
//
// --------------------------------------------------------------------------

// create a palette of the specified size
grPalette* grCreatePalette(uint numentries);

// destroy a specified palette...
void grDestroyPalette(grPalette* pal);

uint grPaletteSize(uint numentries);

// set palette entry #num to r,g,b
void grSetPaletteEntry(uchar entry,uchar r,uchar g,uchar b);

// gets palette entry #num
void grGetPaletteEntry(uchar entry,uchar* r,uchar* g,uchar* b);

// sets palette to palette_ptr, from 'start' to 'end'
void grSetPalette(grPalette* pal,uchar start,uchar end);

// gets palette similar to SetPalette()
void grGetPalette(grPalette* pal,uchar start,uchar end);

void grFadeOutPalette(uchar start,uchar end,uchar timesin);
void grFadeInPalette(uchar start,uchar end,uchar timesin);
void grRotatePalette(uchar start,uchar end);

// --------------------------------------------------------------------------
//
//	MOUSE FUNCTIONS
//
// --------------------------------------------------------------------------

// initialize the mouse driver, placing it on screen
#define grInitMouse			mouse_InstallHandler

// removes the mouse driver
#define grExitMouse			mouse_RemoveHandler

// hide the mouse
#define grHideMouse			mouse_Hide

// show the mouse
#define	grShowMouse			mouse_Show

// --------------------------------------------------------------------------
//
//	GRID FUNCTIONS
//
// --------------------------------------------------------------------------
void grSetGrid(int xoffset,int yoffset,int xboxes,int yboxes,int boxwidth,int boxheight,uchar color);	// set grid parameters
void grDrawGrid();					// draw grid using set params
void grDrawGridElem(int x,int y,uchar color);	// draw element in grid with colors
void grHighlightElem(int x,int y,uchar color);	// highlight element...
grPoint grGridBoxSelected(grPoint p);			// assuming a point, is the box in this area??

// --------------------------------------------------------------------------
//
//	DEBUGGING FUNCTIONS
//
// --------------------------------------------------------------------------
void DBG(int y,char* fmt,...);    			// draw a debugging string
//void clrscr();
//void gotoxy(uchar x,uchar y);


//	Miscellaneous
grImage*	grMergeImage (grImage* over, grImage* under);

//	-------------------------------------------------------------------
//	-------------------------------------------------------------------
//	--- New stuff for GRAPHICS.HPP ------------------------------------
//	-------------------------------------------------------------------
//	-------------------------------------------------------------------
uint	grFontHeight();
uint	grFontCharWidth(char ch);
uint	grFontWidth(char* st);

#endif