//	--------------------------------------------------------------------------
//
//	PALETTES.CPP
//
//	--------------------------------------------------------------------------

#include	"palettes.hpp"

//	-------------------------------------------------------------------------
//
//	FUNC:		PAL_SizeOfPalette
//	DESC:		this should be used outside to malloc the palette's size,
//
//	-------------------------------------------------------------------------
long	PAL_SizeOfPalette(DirString* filename)
	{
	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	long size;

	// seek start
	srcefile->Seek (0);
	// read numentries into size from DOSFile
	srcefile->Read ((char*)&size,sizeof(uchar));

	// close a DOSFile
	delete 		srcefile;
	return size;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PAL_LoadPalette
//	DESC:		Load a palette from a .PAL file
//
//	-------------------------------------------------------------------------
int		PAL_LoadPalette(DirString* filename, grPalette* dest, bool enable_palette)
	{
	int fileError;
	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	// find out the size of the .PAL file
	long buffersize = srcefile->FileSize();

	// seek start
	srcefile->Seek (0);
	// read palette into dest from DOSFile
	fileError = srcefile->Read ((char*)dest,buffersize);

	// if there were no errors
	if (fileError == dosfile_NoError)
		{
		// possibly set up palette?
		if (enable_palette)
			grSetPalette(dest,0,255);
		}

	// close a DOSFile
	delete 		srcefile;

	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PAL_SavePalette
//	DESC:		Save a palette to a .PAL file
//
//	-------------------------------------------------------------------------
int		PAL_SavePalette (DirString* filename, grPalette* srce, long size)
	{
	int fileError = 0;

	// open a DOSFile
	DOSFile* destfile = new DOSFile(filename,dosfile_CREATE|dosfile_WRITEONLY);

	// write palette to DOSFile
	fileError = destfile->Write((char*)srce,size);

	// close a DOSFile
	delete destfile;
	return fileError;
	}



PaletteResourceFile::PaletteResourceFile(DirString* pathname,uint options,long num_resources)
					: ResourceFile(pathname,options,num_resources,resource_PALETTE_RESOURCE)
	{
	}

PaletteResourceFile::PaletteResourceFile(	const char* pathname,uint options,long num_resources)
					: ResourceFile(	pathname,options,num_resources,resource_PALETTE_RESOURCE)
	{
	}

PaletteResourceFile::~PaletteResourceFile()
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ReadResource
//	DESC:       reads a resource of 'size' bytes, and places them in
//				a palette 'buffer'
//
//	-------------------------------------------------------------------------
int PaletteResourceFile::ReadResource(long resource_num,grPalette* buffer)
	{
	// skip out if we're write-only
	if (fileOptions&dosfile_WRITEONLY) return fileError = resource_FileIsWriteOnly;

	// read the resource status
	if ((fileError = ReadResourceIndex( resource_num )) != resource_NoError) return fileError;

	// if the size is zero, there is no said resource
	if (index.size==0) return fileError = resource_ResourceNotFound;

	// seek to the start of the resource
	if ((fileError = Seek( index.location )) != resource_NoError) return fileError;

	// read the resource
	fileError = DOSFile::Read((char*)buffer, index.size );

	// return our last file error
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		AddResource
//	DESC:		Will open a palette file and append it to the end of the
//				resource file.
//				If you want to modify your resource file you have to delete
//				it and rewrite all the resources in, in the correct order
//
//	-------------------------------------------------------------------------
int PaletteResourceFile::AddResource(DirString* srce)
	{
	int fileError = 0;
	long	palette_size;

	// check if pcx file exists then open it
	if (!dir_FileExists(srce)) return dosfile_FileNotFound;
	// open a DOSFile
	DOSFile*		srcefile = new DOSFile(srce,dosfile_READONLY);

	// get size of palette file
	palette_size = srcefile->FileSize();

	// malloc memory for temp_palette where palette file will be loaded
	grPalette* temp_palette = (grPalette*) malloc(palette_size+1);

	// load palette file into temp_palette
	fileError = srcefile->Seek (0);
	fileError = srcefile->Read ((char*)temp_palette,palette_size);

	// write resource to resourcefile
	fileError = WriteResource(NumResources(),palette_size,(char*)temp_palette);

	free(temp_palette);
	// close DOSFile
	delete srcefile;
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		LoadAllResources
//	DESC:		malloc each data item then load each data item of a resource
//
//	-------------------------------------------------------------------------
char** PaletteResourceFile::LoadAllResources()
	{
	// get number of resources to load
	int num_resources = NumResources();

	// malloc memory for whole list of pointers to each resource
	grPalette** datalist = (grPalette**) malloc(sizeof(grPalette*) * num_resources);

	int i;
	for (i=0;i<num_resources;i++)
		{
		// malloc memory for each resource
		datalist[i] = (grPalette*) malloc(SizeOfResource(i)+1);

		// read in a resource
		fileError = ReadResource (i, datalist[i]);
		}
	// return whole list of resources
	return (char**)datalist;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ClearAllResources
//	DESC:		free each data item of a resource
//
//	-------------------------------------------------------------------------
void PaletteResourceFile::FreeAllResources(char** datalist)
	{
	int i;
	for (i=NumResources()-1;i>=0;i--)
		// free each resource
		free(datalist[i]);

	// free whole list of pointers
	free(datalist);
	}
