//	-------------------------------------------------------------------------
//
//	PCX.CPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	PCX utilities
//
//	-------------------------------------------------------------------------

#include		"pcx.hpp"
#include		"dosfile.hpp"

#include		"stdio.h"
#include		"fcntl.h"
#include		"sys/stat.h"

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_ToResource
//	DESC:		convert standard PCX_image to PCX_resource
//				basically discarding the header and palette
//
//	PARAMS:		long buffer_size - size of compressed data ie. buffer
//
//	-------------------------------------------------------------------------
void	PCX_ToResource(PCX_resource* dest, PCX_image* srce ,ulong buffer_size)
	{
	// initialise the resource grImage header
	dest->header.sizeofDecompressed = grImageSize(0,0,srce->header.width,srce->header.height);
	dest->header.width = srce->header.width+1;
	dest->header.height = srce->header.height+1;

	// copy the compressed data from our PCX_image to our PCX_resource
	memcpy((char*)&dest->buffer[0],(char*)&srce->buffer[0],buffer_size);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_ResourceToImage
//	DESC:		decompress a PCX_resource into a grImage
//
//	-------------------------------------------------------------------------
void	PCX_ResourceToImage(grImage* dest, PCX_resource* srce)
	{
	dest->hdr.width = srce->header.width;
	dest->hdr.height = srce->header.height;

	PCX_Decompress(dest,(char*)&srce->buffer[0]);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_ToImage
//	DESC:		decompress a PCX_image into a grImage
//
//	-------------------------------------------------------------------------
void	PCX_ToImage(grImage* dest, PCX_image* srce)
	{
	dest->hdr.width = srce->header.width+1;
	dest->hdr.height = srce->header.height+1;

	PCX_Decompress(dest, (char*)&srce->buffer[0]);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_FromImage
//	DESC:		compress a grImage into a PCX_image
//
//	-------------------------------------------------------------------------
long	PCX_FromImage(PCX_image* dest, grImage* srce)
	{
	// initialise header of PCX_image
	dest->header.manufacturer = 10;
	dest->header.version = 5;
	dest->header.encoding = 1;
	dest->header.bits_per_pixel = 8;
	dest->header.x = 0;
	dest->header.y = 0;
	dest->header.width = srce->hdr.width - 1;// 23x23 / size for a 24x24
	dest->header.height = srce->hdr.height - 1;// 23x23 / size for a 24x24
	dest->header.horz_res = 640;
	dest->header.vert_res = 480;
	dest->header.num_color_planes = 8;
	dest->header.bytes_per_line = srce->hdr.width;

	// compress image data into PCX format
	return PCX_Compress((char*)&dest->buffer[0],srce);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_ResourceFromImage
//	DESC:		compress a grImage into a PCX_resource
//
//	-------------------------------------------------------------------------
long	PCX_ResourceFromImage(PCX_resource* dest, grImage* srce)
	{
	// initialise header of PCX_resource
	dest->header.width = srce->hdr.width;
	dest->header.height = srce->hdr.height;

	// compress image data into PCX format
	return PCX_Compress((char*)&dest->buffer[0],srce);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_Decompress
//	DESC:		decompresses a PCX buffer into grImage data
//
//	-------------------------------------------------------------------------
void	PCX_Decompress(grImage* dest, char* srce)
	{
	long	datacount = 0;
	long	pixelcount = 0;
	long	maxpixelcount = dest->hdr.width * dest->hdr.height;

	uchar	data;
	uint	number_of_pixels;

	while (pixelcount < maxpixelcount )
		{
		// get first piece of data and if this is compressed
		// into an RLE then extract it
		data = (uchar)srce[datacount++];
		if (data >= 192)
			{
			// how many bytes in the run? and get the actual data for the run
			number_of_pixels = data - 192;
			data = (uchar)srce[datacount++];

			// replicate the data into the buffer number_of_pixels times
			while (number_of_pixels > 0)
				{
				dest->data[pixelcount++] = data;
				number_of_pixels--;
				}
			} // end if RLE
		else
			{
			// actual data, just copy straight into buffer
			dest->data[pixelcount++] = data;
			} // end else not RLE
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_Compress
//	DESC:		compress grImage data into a PCX buffer
//
//	NOTE:	My compress function creates data that is not
//			compatible with standard format in XTG or NEOPAINT
//
//	-------------------------------------------------------------------------
long	PCX_Compress(char* dest, grImage* srce)
	{
	long	datacount = 0;
	long	pixelcount = 0;
	long	maxpixelcount = srce->hdr.width * srce->hdr.height;

	uchar	current_color = 0;
	uchar	last_color = srce->data[pixelcount++];
	uchar	number_of_color = 1;
	bool	save = false;

	while (pixelcount < maxpixelcount)
		{
		// load in the next pixel
		current_color = srce->data[pixelcount];
		// if the current and last are same we don't want to save yet
		if (current_color == last_color)
			{
			// another in the row of same colors
			number_of_color++;
			// don't want to save it yet
			save = false;
			// unless we have a row of 63 same colors (192 + 63 = 255)
			if (number_of_color == 63)
				save = true;
			}
		// otherwise we _DO_ want to save this RLE run
		else
			{
			save = true;
			}

		if (save)
			{
			// any value above 191 or any group of pixels with
			// 2 or more the same has to be encoded into an RLE run
			if ((last_color >= 192) || (number_of_color > 1))
				dest[datacount++] = 192 + number_of_color;

			// then save the value of the color
			dest[datacount++] = last_color;
			// reset the number of adjacent colors
			number_of_color = 1;
			}
		// this is the last color of the current RLE run
		last_color = srce->data[pixelcount++];

		// in case the compression isn't really compressing then
		// we'll drop out here
		if (datacount > maxpixelcount-3)
			return pcx_CompressLargerthanImage;
		}
	// for the last RLE run we drop out to here and save it
	if ((last_color >= 192) || (number_of_color > 1))
		dest[datacount++] = 192+number_of_color;

	dest[datacount++] = last_color;
	// few!!! finished compressing grImage into tempbuffer

	// return the size of the whole pcx structure
	return datacount + PCX_HEADER_SIZE + PCX_PALETTE_SIZE;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_ImageSize
//	DESC:		returns the size of a PCX_image as a grImage
//				ie. size = grImageSize(0,0,width,height);
//
//	-------------------------------------------------------------------------
ulong	PCX_ImageSize(DirString* filename)
	{
	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	PCX_imageHeader temphdr;

	// read the pcx header
	srcefile->Seek (0);
	srcefile->Read ((char*) &temphdr, PCX_HEADER_SIZE);

	// calculate the grImage size
	ulong size = grImageSize(0,0,temphdr.width,temphdr.height);

	// close a DOSFile
	delete srcefile;
	return size;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_GetWidthHeight
//	DESC:		will pass back the width and height of a PCX file
//
//	-------------------------------------------------------------------------
int		PCX_GetWidthHeight (DirString* filename, int* width, int* height)
	{
	int fileError = 0;
	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	PCX_imageHeader temphdr;

	// read the pcx header
	fileError = srcefile->Seek (0);
	fileError = srcefile->Read ((char*) &temphdr, PCX_HEADER_SIZE);

	if (fileError != dosfile_NoError)
		return fileError;

	// get width and height
	*width = temphdr.width + 1;
	*height = temphdr.height + 1;

	// close a DOSFile
	delete srcefile;
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_LoadImage
//	DESC:		will load block data into PCX_image* then decompress
//				it into grImage*
//                                                       
//	-------------------------------------------------------------------------
int		PCX_LoadImage (DirString* filename, grImage* dest, bool enable_palette)
	{
	int fileError = 0;
	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	// get the size of the file and malloc a temporary storage
	long buffersize = srcefile->FileSize();
	PCX_image*	tempimg = (PCX_image*) malloc(buffersize+1);

	// read the pcx file
	fileError = srcefile->Seek (0);
	fileError = srcefile->Read ((char*)&tempimg->header, PCX_HEADER_SIZE);

	fileError = srcefile->Seek (buffersize - PCX_PALETTE_SIZE);
	fileError = srcefile->Read ((char*)&tempimg->pal[0], PCX_PALETTE_SIZE);

	fileError = srcefile->Seek (PCX_HEADER_SIZE);
	fileError = srcefile->Read ((char*)&tempimg->buffer[0], buffersize - PCX_HEADER_SIZE - PCX_PALETTE_SIZE);

	// if there were no errors
	if (fileError == dosfile_NoError)
		{
		// convert and decompress the PCX_image into a grImage
		PCX_ToImage(dest,tempimg);

		// possibly set up palette?
		if (enable_palette)
			{
			// create a temporary palette
			grPalette* pal = grCreatePalette(256);
			// copy it into PCX_image
			memcpy((char*)&pal->data[0],(char*)&tempimg->pal[0],PCX_PALETTE_SIZE);
			// set current system palette to loaded palette

			int i;
			// adjust the palette
			for (i=0; i<PCX_PALETTE_SIZE; i++)
				pal->data[i] = pal->data[i] / 4;

			grSetPalette(pal,0,255);
			// free temporary palette
			grDestroyPalette(pal);
			}
		}

	free( tempimg );
	// close a DOSFile
	delete srcefile;
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_SaveImage
//	DESC:		will compress an grImage* into PCX_image* and
//				write block data to filename
//
//	-------------------------------------------------------------------------
int		PCX_SaveImage (DirString* filename, grImage* srce)
	{
	int fileError = 0;

	PCX_image*	tempimg = (PCX_image*) malloc(PCX_HEADER_SIZE+sizeof(srce)+PCX_PALETTE_SIZE);

	// compress srce taking its size or an error
	long	buffersize = PCX_FromImage(tempimg,srce);

	// if compress size was not larger than original size
	if (buffersize > pcx_CompressSizeOk)
		{
		// open a DOSFile
		DOSFile*	destfile = new DOSFile(filename,dosfile_CREATE|dosfile_WRITEONLY);

		// create a temporary palette
		grPalette* pal = grCreatePalette(256);
		// get current palette from system
		grGetPalette(pal,0,256);

		int i;
		// adjust the palette
		for (i=0; i<PCX_PALETTE_SIZE; i++)
			pal->data[i] = pal->data[i] * 4;

		// copy it into PCX_image
		memcpy((char*)&tempimg->pal[0],(char*)&pal->data[0],PCX_PALETTE_SIZE);
		// free temporary palette
		grDestroyPalette(pal);

		// write the temporary pcx grImage to DOSFile
		fileError = destfile->Write ((char*) &tempimg->header, PCX_HEADER_SIZE);
		fileError = destfile->Write ((char*) &tempimg->buffer[0], buffersize - PCX_HEADER_SIZE - PCX_PALETTE_SIZE);
		fileError = destfile->Write ((char*) &tempimg->pal[0], PCX_PALETTE_SIZE);

		// close a DOSFile
		delete		destfile;
		}
	else
		// set fileError to say that grImage was larger than original
		fileError = pcx_CompressLargerthanImage;

	free (tempimg);
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_LoadPaletteFromPCX
//	DESC:		Load a palette from a .PCX file
//
//	-------------------------------------------------------------------------
grPalette*	PCX_LoadPalette(DirString* filename, bool enable_palette)
	{
	// create a palette to store information in
	grPalette* pal = grCreatePalette(256);

	// open a DOSFile
	DOSFile*	srcefile = new DOSFile(filename,dosfile_READONLY);

	// seek to 768 from end of file
	srcefile->Seek (srcefile->FileSize() - PCX_PALETTE_SIZE);
	// read palette into dest from DOSFile
	srcefile->Read ((char*)&pal->data[0], PCX_PALETTE_SIZE);

	int i;
	// adjust the palette
	for (i=0; i<PCX_PALETTE_SIZE; i++)
		pal->data[i] = pal->data[i] / 4;

	// possibly set up palette?
	if (enable_palette)
		grSetPalette(pal,0,255);

	// close a DOSFile
	delete srcefile;
	return pal;
	}
/*
//	-------------------------------------------------------------------------
//
//	FUNC:		PCX_MakeSpriteFromPCXs
//	DESC:		will create a sprite then load a number of PCX files
//				as grImages into that sprite
//
//	PARAMS:		filelist - list of paths for each PCX file
//				number_frames - how many PCX files / frames
//
//	-------------------------------------------------------------------------
sprite* MakeSpriteFromPCXs(DirString** filelist, int number_frames)
	{
	int		width,
			height;

	// get width and height of first PCX image
	PCX_GetWidthHeight(filelist[0], &width, &height);

	sprite* spr = grCreateSprite(number_frames, width, height);

	// load each pcx file into a grImage and make it a part of the resource
	for (int i=0;i<number_frames;i++)
		{
		// load PCX image as grImage into a sprite frame
		PCX_LoadImage(filelist[i],spr->frame[i],false);

		// confirm in writing
		printf("\nADDING    %s    ",filelist[i]->Contents());
		}

	// return the sprite
	return spr;
	}
*/