//	-------------------------------------------------------------------------
//
//	RESOURCE.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file implements the Resource object which is used to
//					access a pseudo-random-access file containing variable
//					length records (resources)
//
//	-------------------------------------------------------------------------

#include "resource.hpp"

#include "process.h"
#include "fcntl.h"
#include "dos.h"
#include "sys\stat.h"
#include "errno.h"

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for File
//	DESC:       constructs an object using an external DirString specification
//
//	-------------------------------------------------------------------------
ResourceFile::ResourceFile(DirString* pathname,uint options,long num_resources,uint type) : DOSFile(pathname,options)
	{
	// if we are creating the resource file
	if (options&resource_CREATE)
		{
		// create the blank header
		header.type = type;
		header.max_resources = num_resources;
		header.num_resources = 0;

		// and store it in the file
		WriteHeader();

		// create the blank indexes & store them
		for (int i=0;i<num_resources;i++)
			{
			index.location = 0;
			index.size = 0;
			WriteResourceIndex(i);
			}
		}
	else
		// simply load relevant information
		ReadHeader();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for ResourceFile
//	DESC:       constructs an object using an internal DirString specification
//
//	-------------------------------------------------------------------------
ResourceFile::ResourceFile(const char* pathname,uint options,long num_resources,uint type) : DOSFile(pathname,options)
	{
	// if we are creating the resource file
	if (options&resource_CREATE)
		{
		// create the blank header
		header.type = type;
		header.max_resources = num_resources;
		header.num_resources = 0;

		// and store it in the file
		WriteHeader();

		// create the blank indexes & store them
		for (int i=0;i<num_resources;i++)
			{
			index.location = 0;
			index.size = 0;
			WriteResourceIndex(i);
			}
		}
	else
		// simply load relevant information
		ReadHeader();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		DESTRUCTOR for ResourceFile
//	DESC:		clears up the file on delete...
//
//	-------------------------------------------------------------------------
ResourceFile::~ResourceFile()
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ReadHeader()
//	DESC:		reads the resource file header
//
//	-------------------------------------------------------------------------
int ResourceFile::ReadHeader()
	{
	// skip out if file isn't open
	if (!fileIsOpen) return fileError = resource_FileNotOpen;

	// seek to the start of the file
	if ((fileError = Seek(0)) != resource_NoError) return fileError;

	// and read the header
	return fileError = DOSFile::Read((char*)&header,sizeof(ResourceHeader));
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		WriteHeader()
//	DESC:		writes the resource file header
//
//	-------------------------------------------------------------------------
int ResourceFile::WriteHeader()
	{
	// skip out if file isn't open
	if (!fileIsOpen) return fileError = resource_FileNotOpen;

	// seek to the start of the file
	if ((fileError = Seek(0)) != resource_NoError) return fileError;

	// and read the header
	return fileError = DOSFile::Write((char*) &header,sizeof(ResourceHeader));
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ReadResourceIndex()
//	DESC:		reads the location & size pertaining to a particular resource
//
//	-------------------------------------------------------------------------
int ResourceFile::ReadResourceIndex(long resource_num)
	{
	// skip out if file isn't open
	if (!fileIsOpen) return fileError = resource_FileNotOpen;

	// skip out if the resource is out of bounds
	if (resource_num>header.num_resources) return fileError = resource_ResourceNotFound;

	// seek to the start of the file + offsets
	if ((fileError = Seek(0 + sizeof(ResourceHeader)
							+ sizeof(ResourceIndex) * resource_num
							)) != resource_NoError) return fileError;

	// and read the header
	return fileError = DOSFile::Read((char*) &index,sizeof(ResourceIndex));
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		WriteResourceIndex()
//	DESC:		reads the location & size pertaining to a particular resource
//
//	-------------------------------------------------------------------------
int ResourceFile::WriteResourceIndex(long resource_num)
	{
	// skip out if file isn't open
	if (!fileIsOpen) return fileError = resource_FileNotOpen;

	// skip out if the resource is out of bounds
	if (resource_num>header.max_resources) return fileError = resource_ResourceNotFound;

	// seek to the start of the file + offsets
	if ((fileError = Seek(0 + sizeof(ResourceHeader)
							+ sizeof(ResourceIndex) * resource_num
							)) != resource_NoError) return fileError;

	// and read the header
	return fileError = DOSFile::Write((char*) &index,sizeof(ResourceIndex));
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		NumResources()
//	DESC:       returns the number of resources in the resourcefile
//				(at present)
//
//	-------------------------------------------------------------------------
long ResourceFile::NumResources()
	{
	// read the resource header caJUSTse
	if (ReadHeader() != resource_NoError) return fileError;

	// and return the number of records in the file
	return header.num_resources;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ResourceSize()
//	DESC:       returns the size of a specified resource
//
//	-------------------------------------------------------------------------
long ResourceFile::ResourceSize(long resource_num)
	{
	if (ReadResourceIndex(resource_num)==resource_NoError) return index.size;
	return fileError = resource_ResourceNotFound;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ReadResource()
//	DESC:       reads a resource of 'size' bytes and places them in 'buffer'
//
//	-------------------------------------------------------------------------
int ResourceFile::ReadResource(long resource_num,char* buffer)
	{
	// skip out if we're write-only
	if (fileOptions&dosfile_WRITEONLY) return fileError = resource_FileIsWriteOnly;

	// read the resource status
	if ((fileError = ReadResourceIndex( resource_num )) != resource_NoError) return fileError;

	// if the size is zero, there is no said resource
	if (index.size==0) return fileError = resource_ResourceNotFound;

	// seek to the start of the resource
	if ((fileError = Seek( index.location )) != resource_NoError) return fileError;

	// read the resource
	fileError = DOSFile::Read(buffer, index.size );

	// decompress the resource does nothing, but should be included in derived resources for decompression
//	if (fileError==resource_NoError) DecompressResource();

	// return our last file error
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		WriteResouce()
//	DESC:       writes a resource of 'size' bytes starting at 'buffer'
//
//	-------------------------------------------------------------------------
int ResourceFile::WriteResource(long resource_num,long size,char* buffer)
	{
//	DBG(300,"IN WRITERESOURCE");
	// skip out if we're read-only
	if (fileOptions&dosfile_READONLY) return fileError = resource_FileIsReadOnly;

//	DBG(310,"READING RESOURCE INDEX");
	// read the resource status
	if ((fileError = ReadResourceIndex( resource_num )) != resource_NoError) return fileError;

	// if we have a zero size, we have not yet allocated the resource
	if (index.size == 0)
		{
//		DBG(320,"ATTEMPTING TO SEEK TO END_OF_FILE");
		// seek the starting location of the resource
		Seek(FileSize()); // ignore errors

		// and set up our data
		index.location = FilePos();
		index.size = size;
		}
	else
		{
//		DBG(320,"SEEKING TO START OF RESOURCE");
		// check whether or not the size is okay
		if (size!=index.size) return fileError = resource_ResourceAlreadyUsed;

		// seek the starting location of the resource
		if ((fileError = Seek( index.location )) != resource_NoError) return fileError;
		}

//	DBG(350,"ABOUT TO COMPRESS THE RESOURCE");
	// compress the resource does nothing, but should be included in derived resources for compression
	if (fileError==resource_NoError) CompressResource();

//	DBG(360,"FINISHED COMPRESSING THE RESOURCE");
//	DBG(370,"WRITE_RESOURCE:: WRITING %ld bytes TO THE RESOURCE",size);

	// and now write the resource
	if ((fileError = DOSFile::Write(buffer,size)) != resource_NoError) return fileError;

//	DBG(380,"FINISHED WRITING THE RESOURCE, err = %d",fileError);

	// update the relevent index
	WriteResourceIndex( resource_num );

	// update the resource header
	ReadHeader();
	header.num_resources++;
	WriteHeader();

	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		SizeOfDecompressedResource
//	DESC:		return the size of the malloc where resource is being
//				loaded/decompressed into
//
//	-------------------------------------------------------------------------
long ResourceFile::SizeOfResource(long resource_num)
	{
	// read the resource index
	if ((fileError = ReadResourceIndex( resource_num )) != resource_NoError) return fileError;

	// if the size is zero, there is no said resource
	if (index.size==0) return fileError = resource_ResourceNotFound;

	// seek to the start of the resource
	if ((fileError = Seek( index.location )) != resource_NoError) return fileError;

	// for undefined resources or resources that need no decompression
	// the decompressed size is the same as resource size
	long size = index.size;

	return size;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		LoadAllResources
//	DESC:		malloc each data item then load each data item of a resource
//
//	-------------------------------------------------------------------------
char** ResourceFile::LoadAllResources()
	{
	// get number of resources to load
	int num_resources = NumResources();

	// malloc memory for whole list of pointers
	char** datalist = (char**) malloc(sizeof(char*) * num_resources);

	int i;
	for (i=0;i<num_resources;i++)
		{
		// malloc memory for each resource
		datalist[i] = (char*) malloc(SizeOfResource(i)+1);

		// read in a resource
		fileError = ReadResource (i, (char*)datalist[i]);
		// on error
		if (fileError)
			{
//			grDrawString(0,400,"could not read resource error #%d",fileError);
			}
		}
	// return pointer to list of resources
	return datalist;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ClearAllResources
//	DESC:		free each data item of a resource
//
//	-------------------------------------------------------------------------
void ResourceFile::FreeAllResources(char** datalist)
	{
	int i;
	for (i=NumResources()-1;i>=0;i--)
		// free each resource
		free(datalist[i]);

	// free whole list
	free (datalist);
	}
